/**
 * @file   ${project_camel_name}.cpp
 * @author ${project_author}
 * @brief  This file defines the ${project_camel_name} class
 */

#include "${project_camel_name}.h"


namespace ${project_name}
{

${project_camel_name}::${project_camel_name}()
{

}

${project_camel_name}::~${project_camel_name}()
{

}

} // namespace ${project_name}
