/**
 * @file   ${project_camel_name}.hpp
 * @author ${project_author}
 * @brief  This file declares and defines the ${project_camel_name} class
 */

#pragma once


namespace ${project_name}
{

/// Main class for ${project_title}
class ${project_camel_name}
{
public:
    /// @brief Construct ${project_camel_name}
    ${project_camel_name}() {}
    virtual ~${project_camel_name}() {}
};

} // namespace ${project_name}
