/**
 * @file   main.cpp
 * @author ${project_author}
 * @brief  This file contains the main function for this project
 */

#include "${project_camel_name}.h"


int main(int argc, char* argv[])
{
    using namespace ${project_name};

    ${project_camel_name} ${project_name}; ///< ${project_camel_name} instance

    return 0;
}
